"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var templates_1 = require("./templates");
var commander = require('commander');
var ejs = require('ejs');
var fs = require('fs');
var chalk = require('chalk');
var path = require('path');
var _ = require('lodash');
var tool = require('antlr4-tool');
var parserUtil = require('antlr4-tool/dist/antlr-core/parser-util');
var mkdir = require('mkdir-recursive').mkdirSync;
var grammars;
var results = commander
    .name('antlr4-editor')
    .option('-r --root-rule <rule>', 'The Root Rule of the Grammar (required)')
    .option('-o --output-path <directory>', 'The Source Output Directory', 'src/editor')
    .option('-p --editor-platform [platform]', 'The Editor Platform', 'codemirror')
    .option('-m --editor-module [module]', '', 'antlr4-editor')
    .arguments('[grammars...]')
    .action(function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    grammars = args[0];
})
    .parse(process.argv);
if (_.isNil(results['rootRule'])) {
    console.log(chalk.red('Please specify a root rule'));
    results.help();
}
var rootRule = results['rootRule'];
var editorModule = results['editorModule'];
var editorPlatform = results['editorPlatform'];
var outputDirectory = results['outputPath'];
var compilerOpts = {
    grammarFiles: grammars,
    outputDirectory: outputDirectory,
    language: 'JavaScript'
};
var compiledResults = tool.compile(compilerOpts);
_.each(compiledResults, function (files, grammar) {
    var parsers = _.filter(files, function (file) { return file.includes('Parser.js'); });
    var lexers = _.filter(files, function (file) { return file.includes('Lexer.js'); });
    files.forEach(function (f) { return console.log("Generated " + chalk.blue.underline(f)); });
    generateFile(outputDirectory + "/index.js", templates_1.templates.index, {});
    generateFile(outputDirectory + "/index.d.ts", templates_1.templates.indexTs, { editorModule: editorModule });
    generateFile(outputDirectory + "/test-editor-app.js", templates_1.templates.app, {});
    if (parsers.length > 0) {
        var parser = parserUtil.readParser(grammar, parsers[0]);
        generateFile(outputDirectory + "/editor.js", templates_1.templates.editor, createTemplateCtx(grammar, parser));
        if (lexers.length > 0) {
            var lexer = parserUtil.readLexer(grammar, lexers[0]);
            generateFile(outputDirectory + "/test.js", templates_1.templates.test, createTemplateCtx(grammar, parser, lexer));
        }
    }
    mkdir(outputDirectory + "/css/");
    _.each(loadCSSFiles(editorPlatform), function (content, css) {
        var filePath = outputDirectory + "/css/" + css;
        fs.writeFileSync(filePath, content);
        console.log("Generated " + chalk.blue.underline(path.resolve(filePath)));
    });
    {
        var ctx = createTemplateCtx(grammar);
        ctx.appPath = './test.js';
        generateFile(outputDirectory + "/index.html", templates_1.templates.indexHtml, ctx);
    }
});
function createTemplateCtx(grammar, parser, lexer) {
    var templateContext = {};
    templateContext.grammar = grammar;
    templateContext.LexerClass = grammar + "Lexer";
    templateContext.ParserClass = grammar + "Parser";
    templateContext.editorModule = editorModule;
    templateContext.rootRule = rootRule;
    templateContext.editorPlatform = editorPlatform;
    templateContext['_'] = _;
    if (parser) {
        var rulesNames = _.filter(parser.ruleNames, function (name) { return name; });
        templateContext.rulesNames = rulesNames;
    }
    if (lexer) {
        var rulesNames = _.filter(lexer.ruleNames, function (name) { return name; });
        templateContext.lexerRulesNames = rulesNames;
    }
    return templateContext;
}
function generateFile(filePath, templateStr, ctx) {
    var content = ejs.render(templateStr, ctx);
    fs.writeFileSync(filePath, content);
    console.log("Generated " + chalk.blue.underline(path.resolve(filePath)));
}
function loadCSSFiles(editorImplementation) {
    var list = [];
    var map = {};
    list[0] = editorImplementation + "-all-themes.css";
    list[1] = editorImplementation + "-editor.css";
    list[2] = editorImplementation + "-icons.css";
    list.forEach(function (val) {
        map[val] = fs.readFileSync(__dirname + "/../css/" + val).toString();
    });
    return map;
}
//# sourceMappingURL=cli.js.map