"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var codemirror_editor_1 = require("../codemirror-editor");
var _ = require("lodash");
var generic_completion_popup_1 = require("../../../completion/generic-completion-popup");
var placeholder_auto_complete_event_1 = require("../../../event/placeholder-auto-complete-event");
var CodeMirrorPlaceholder = /** @class */ (function () {
    function CodeMirrorPlaceholder(editor, domContainer, range) {
        this.editor = editor;
        this.domContainer = domContainer;
        this.range = range;
        var el = document.createElement('span');
        el.contentEditable = 'true';
        el.className = 'antlr-placeholder-element';
        this.inputElement = el;
        this.domContainer.appendChild(el);
        this.domContainer.style.display = 'inline-block';
        this.domContainer.style.position = 'relative';
        this.mark = this.editor.editorImplementation.getDoc()
            .markText({ ch: range[0].column, line: range[0].line }, {
            ch: range[1].column,
            line: range[1].line
        }, { replacedWith: this.domContainer });
    }
    CodeMirrorPlaceholder.prototype.addChangeListener = function (listener) {
        var _this = this;
        this.inputElement.addEventListener('input', function (event) {
            event.stopPropagation();
            listener(new placeholder_auto_complete_event_1.PlaceHolderAutoCompleteEvent(_this, _this.inputElement.innerText, _this.editor));
        });
    };
    CodeMirrorPlaceholder.prototype.getTextRange = function () {
        return this.range;
    };
    CodeMirrorPlaceholder.prototype.showCompletions = function (completions) {
        var _this = this;
        this.clearCompletions();
        var popup = new generic_completion_popup_1.GenericCompletionPopup(this.domContainer, this.editor);
        this.currentCompletionPopup = popup;
        setTimeout(function () {
            popup.singleCompletionCssClass = codemirror_editor_1.HINT_CLASS;
            popup.completionsCssClass = codemirror_editor_1.HINTS_CLASS;
            popup.activeCompletionCssClass = codemirror_editor_1.ACTIVE_HINT_CLASS;
            popup.showHints(_this.domContainer, completions);
            popup.afterSelection = function (completion) {
                _this.range = completion.range;
                _this.editor.focus();
            };
        });
        return popup;
    };
    CodeMirrorPlaceholder.prototype.getDomElement = function () {
        return this.domContainer;
    };
    CodeMirrorPlaceholder.prototype.getInputElement = function () {
        return this.inputElement;
    };
    CodeMirrorPlaceholder.prototype.setRangeWithText = function (text) {
        this.range = this.editor.replaceRange(this.range, text);
    };
    CodeMirrorPlaceholder.prototype.setPlaceHolderText = function (text) {
        this.placeHolderText = text;
        this.inputElement.setAttribute('placeholder', this.placeHolderText);
    };
    CodeMirrorPlaceholder.prototype.getPlaceHolderText = function () {
        return this.placeHolderText;
    };
    CodeMirrorPlaceholder.prototype.focus = function () {
        var _this = this;
        setTimeout(function () { return _this.inputElement.focus(); }, 20);
    };
    CodeMirrorPlaceholder.prototype.exists = function () {
        return this.getDomElement().contains(this.domContainer);
    };
    CodeMirrorPlaceholder.prototype.enableDefaultKeyEventHandlers = function () {
        var _this = this;
        this.inputElement.addEventListener('keypress', function (e) { return e.code !== 'Enter'; });
        this.inputElement.addEventListener('keypress', function (event) {
            event.stopImmediatePropagation();
        });
        this.inputElement.addEventListener('keyup', function (event) {
            event.stopImmediatePropagation();
        });
        this.inputElement.addEventListener('keydown', function (event) {
            event.stopImmediatePropagation();
            switch (event.code) {
                case 'Backspace':
                case 'Delete':
                    if (_.isEmpty(_this.inputElement.innerText)) {
                        event.preventDefault();
                        _this.clearMark();
                        _this.editor.focus();
                    }
                    return;
                case 'Escape':
                    event.preventDefault();
                    _this.clearMark();
                    _this.editor.focus();
                    return;
                case 'Enter':
                    event.preventDefault();
                    if (_this.currentCompletionPopup && _this.currentCompletionPopup.hasHints()) {
                        _this.currentCompletionPopup.select();
                    }
                    else {
                        _this.setRangeWithText(_this.inputElement.innerText);
                    }
                    return;
                case 'ArrowLeft':
                    if (_.isEmpty(_this.inputElement.innerText)) {
                        var cursor = _this.range[0];
                        _this.editor.focus();
                        _this.inputElement.style.left = '';
                        _this.editor.setCursorPosition(cursor);
                    }
                    return;
                case 'ArrowRight':
                    if (_.isEmpty(_this.inputElement.innerText)) {
                        var position = { column: _this.range[1].column, line: _this.range[1].line };
                        _this.editor.focus();
                        _this.editor.setCursorPosition(position);
                    }
                    return;
                case 'ArrowDown':
                    _this.chooseNextCompletion();
                    return;
                case 'ArrowUp':
                    _this.choosePrevCompletion();
                    return;
            }
        });
    };
    CodeMirrorPlaceholder.prototype.choosePrevCompletion = function () {
        if (this.currentCompletionPopup) {
            this.currentCompletionPopup.choosePrevCompletion();
        }
    };
    CodeMirrorPlaceholder.prototype.chooseNextCompletion = function () {
        if (this.currentCompletionPopup) {
            this.currentCompletionPopup.chooseNextCompletion();
        }
    };
    CodeMirrorPlaceholder.prototype.clearMark = function () {
        if (this.mark) {
            this.mark.clear();
            this.mark = undefined;
        }
    };
    CodeMirrorPlaceholder.prototype.clearCompletions = function () {
        if (!_.isNil(this.currentCompletionPopup)) {
            this.currentCompletionPopup.removeCompletions();
        }
    };
    return CodeMirrorPlaceholder;
}());
exports.CodeMirrorPlaceholder = CodeMirrorPlaceholder;
//# sourceMappingURL=codemirror-placeholder.js.map